using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// GLGeometry (Dankook University Kyoung Shin Park 2023/09/21)
public static class GLGeometry
{
    #region Line
    /// <summary>
    /// Draws a line between 2 points. 
    /// </summary>
    public static void Line(Color color, Vector3 p1, Vector3 p2)
    {
        GL.PushMatrix();
        GL.Begin(GL.LINES);
        GL.Color(color);

        // Vertices
        GL.Vertex(p1);
        GL.Vertex(p2);

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region FilledTriangle
    /// <summary>
    /// Draws a filled triangle. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void FilledTriangle(Color c1, Color c2, Color c3, Vector3 p1, Vector3 p2, Vector3 p3)
    {
        GL.PushMatrix();
        GL.Begin(GL.TRIANGLES);

        GL.Color(c1);
        GL.Vertex(p1);

        GL.Color(c2);
        GL.Vertex(p2);

        GL.Color(c3);
        GL.Vertex(p3);

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Rect
    /// <summary>
    /// Draws a filled quad. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void Rect(Color c1, Color c2, Color c3, Color c4, Vector3 p1, Vector3 p2, Vector3 p3, Vector3 p4)
    {
        GL.PushMatrix();
        GL.Begin(GL.QUADS);

        GL.Color(c1);
        GL.Vertex(p1);
        GL.Color(c2);
        GL.Vertex(p2);
        GL.Color(c3);
        GL.Vertex(p3);
        GL.Color(c4);
        GL.Vertex(p4);

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Quad
    /// <summary>
    /// Draws a quad. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void Quad(Color c, Vector3 p, Vector3 n, Vector3 u, int width, int height)
    {
        u.Normalize();
        n.Normalize();
        Vector3 left = Vector3.Cross(u, n); 
        Vector3 uppercenter = (u * height/2.0f) + p;
        Vector3 tl = uppercenter + (left * width/2.0f);
        Vector3 tr = uppercenter - (left * width/2.0f);
        Vector3 bl = tl - (u * height);
        Vector3 br = tr - (u * height);
        //Debug.Log("tl=" + tl + " tr=" + tr + " bl=" + bl + " br=" + br);

        GL.PushMatrix();
        GL.Begin(GL.QUADS);

        GL.Color(c);
        GL.Vertex(bl);
        GL.Vertex(tl);
        GL.Vertex(tr);
        GL.Vertex(br);

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Circle
    /// <summary>
    /// Draws a circle. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void Circle(Color color, Vector3 center, float radius, int segments)
    {
        GL.PushMatrix();    
        GL.Begin(GL.LINE_STRIP);
        
        GL.Color(color);
        //Debug.Log("Circle Begin");
        float deltaTheta = -2.0f * Mathf.PI / segments;
        for (int i = 0; i <= segments; i++)
        {
            float theta = i * deltaTheta;
            float x = center.x + radius * Mathf.Cos(theta);
            float y = center.y + radius * Mathf.Sin(theta);
            float z = center.z;
            GL.Vertex(new Vector3(x, y, z));
            //Debug.Log("Vertex x=" + x + " y=" + y);
        }
        //Debug.Log("Circle End");

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Cylinder
    /// <summary>
    /// Draws a cylinder. (Unity LHS x+ right y+ up z+ inside)  
    /// </summary>
    public static void Cylinder(Color color, Vector3 center, float radius, float height, int segments)
    {
        GL.PushMatrix();
        GL.Begin(GL.TRIANGLE_STRIP);
        GL.Color(color);

        float deltaTheta = 2.0f * Mathf.PI / segments;
        for (int i = 0; i <= segments; i++)
        {
            float theta = i * deltaTheta;
            float x = center.x + radius * Mathf.Cos(theta);
            float y = center.y + -height/2;
            float z = center.z + radius * Mathf.Sin(theta);
            GL.Vertex(new Vector3(x, y, z));
            y = center.y + height/2;
            GL.Vertex(new Vector3(x, y, z));
        }
        
        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Sphere
    /// <summary>
    /// Draws a sphere. 
    /// </summary>
    public static void Sphere(Color color, Vector3 center, float radius, int stacks, int slices)
    {
        GL.PushMatrix();
        GL.Begin(GL.LINE_STRIP);
        GL.Color(color);

        float lonstep = Mathf.PI / stacks;
        float latstep = Mathf.PI / slices;
        for (float lon = 0.0f; lon <= 2*Mathf.PI; lon += lonstep)
        {
            for (float lat = 0.0f; lat <= Mathf.PI + latstep; lat += latstep)
            {
                float x = center.x + radius * Mathf.Cos(lon) * Mathf.Sin(lat);
                float y = center.y + radius * Mathf.Sin(lon) * Mathf.Sin(lat);
                float z = center.z + radius * Mathf.Cos(lat);
                GL.Vertex(new Vector3(x, y, z));
                x = center.x + radius * Mathf.Cos(lon + lonstep) * Mathf.Sin(lat);
                y = center.y + radius * Mathf.Sin(lon + lonstep) * Mathf.Sin(lat);
                z = center.z + radius * Mathf.Cos(lat);
                GL.Vertex(new Vector3(x, y, z));
            }
        }
        
        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Cube
    /// <summary>
    /// Draws a cube. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void Cube(Color color, Vector3 center, float size)
    {
        Vector3 fll = center + new Vector3(-size, -size,  -size);
        Vector3 flr = center + new Vector3( size, -size,  -size);
        Vector3 ful = center + new Vector3(-size,  size,  -size);
        Vector3 fur = center + new Vector3(size, size, -size);
        Vector3 bll = center + new Vector3(-size, -size,  size);
        Vector3 blr = center + new Vector3( size, -size,  size);
        Vector3 bul = center + new Vector3(-size,  size,  size);
        Vector3 bur = center + new Vector3(size, size, size);

        GL.PushMatrix();
        GL.Begin(GL.QUADS);
        GL.Color(color);

        // front face
        GL.Vertex(fll);
        GL.Vertex(ful);
        GL.Vertex(fur);
        GL.Vertex(flr);
        
        // back face
        GL.Vertex(blr);
        GL.Vertex(bur);
        GL.Vertex(bul);
        GL.Vertex(bll);
       
        // left face
        GL.Vertex(bll);
        GL.Vertex(bul);
        GL.Vertex(ful);
        GL.Vertex(fll);

        // right face
        GL.Vertex(flr);
        GL.Vertex(fur);
        GL.Vertex(bur);
        GL.Vertex(blr);

        // top face
        GL.Vertex(ful);
        GL.Vertex(bul);
        GL.Vertex(bur);
        GL.Vertex(fur);

        // bottom face
        GL.Vertex(blr);
        GL.Vertex(bll);
        GL.Vertex(fll);
        GL.Vertex(flr);

        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Torus
    /// <summary>
    /// Draws a torus. 
    /// </summary>
    public static void Torus(Color color, float radius, float tubeRadius, int rings, int sides)
    {
        GL.PushMatrix();
        GL.Begin(GL.LINE_STRIP);
        GL.Color(color);

        float theta = 2 * Mathf.PI / rings; // torus outer radius
        float phi = 2 * Mathf.PI / sides;   // tube inner radius
        for (float v = 0.0f; v <= 2*Mathf.PI; v += phi)
        {
            for (float u = 0.0f; u <= 2*Mathf.PI; u += theta)
            {
                float x = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Cos(u);
                float y = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Sin(u);
                float z = tubeRadius * Mathf.Sin(v);
                GL.Vertex(new Vector3(x, y, z)); // vertex 0
                x = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Cos(u + theta);
                y = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Sin(u + theta);
                z = tubeRadius * Mathf.Sin(v); // vertex 1
                GL.Vertex(new Vector3(x, y, z));
                x = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Cos(u + theta);
                y = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Sin(u + theta);
                z = tubeRadius * Mathf.Sin(v + phi);
                GL.Vertex(new Vector3(x, y, z)); // vertex 2
                x = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Cos(u + theta);
                y = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Sin(u + theta);
                z = tubeRadius * Mathf.Sin(v + phi);
                GL.Vertex(new Vector3(x, y, z)); // vertex 2
                x = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Cos(u);
                y = (radius + tubeRadius * Mathf.Cos(v + phi)) * Mathf.Sin(u);
                z = tubeRadius * Mathf.Sin(v); // vertex 3
                GL.Vertex(new Vector3(x, y, z));
                x = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Cos(u);
                y = (radius + tubeRadius * Mathf.Cos(v)) * Mathf.Sin(u);
                z = tubeRadius * Mathf.Sin(v);
                GL.Vertex(new Vector3(x, y, z)); // vertex 0
            }
        }
        
        GL.End();
        GL.PopMatrix();
    }
    #endregion

    #region Parallelepiped 
    /// <summary>
    /// Draws a parallelepiped. (Unity LHS x+ right y+ up z+ inside) CW
    /// </summary>
    public static void Parallelepiped(Color color, Vector3 p, Vector3 u, Vector3 v, Vector3 w)
    {
        Vector3 fll = p;
        Vector3 flr = p + u;
        Vector3 fur = p + u + v;
        Vector3 ful = p + v;
        Vector3 bll = p + w;
        Vector3 blr = p + u + w;
        Vector3 bur = p + u + v + w;
        Vector3 bul = p + v + w;

        GL.PushMatrix();
        GL.Begin(GL.QUADS);
        GL.Color(color);

        // front face
        GL.Vertex(fll);
        GL.Vertex(ful);
        GL.Vertex(fur);
        GL.Vertex(flr);
        
        // back face
        GL.Vertex(blr);
        GL.Vertex(bur);
        GL.Vertex(bul);
        GL.Vertex(bll);
       
        // left face
        GL.Vertex(bll);
        GL.Vertex(bul);
        GL.Vertex(ful);
        GL.Vertex(fll);

        // right face
        GL.Vertex(flr);
        GL.Vertex(fur);
        GL.Vertex(bur);
        GL.Vertex(blr);

        // top face
        GL.Vertex(ful);
        GL.Vertex(bul);
        GL.Vertex(bur);
        GL.Vertex(fur);

        // bottom face
        GL.Vertex(blr);
        GL.Vertex(bll);
        GL.Vertex(fll);
        GL.Vertex(flr);

        GL.End();
        GL.PopMatrix();
    }
    #endregion
}
